'-------------------------------------------------------------------------------
'   DDOC_DEMO.BAS
'   PowerBASIC v/7 'DDT' demo program to show how to print documents with
'   logo and watermark.
'   Author: Michael Mattias Racine WI USA.
'   Public domain November 2003
'   Compiler Used: PowerBASIC Inc PB/Windows v 7.0
'   --------
'   Logo file "logo_file.bmp" bearing the Tal Systems Inc logo copyright 2001 Tal Systems Inc.
'   Racine WI USA. All rights reserved. The Tal Systems logo may not be used by anyone for
'   any purpose other than this demonstration without the express written permission
'   of Tal Systems Inc. Racine WI. Inquiries by email to info@talsystems.com
'   Header file DDOC_P32 copyright 1995-2001 Don Dickinson. Used with permission. Not supplied
'   with demo.
'   ---------------------
'   November 2002
'   ---------------------------
'   Usage: Start program, browse for a file. Click Print to start DDOC interactive
'   viewer printer.
'   File to be printed should be ordinary text.
'   To force a page change, make a line of text the literal ".PAGE."

' REQUIRES:
' To run: Ddoc32.DLL and Ddoc.exe must be available to program. Supplied with this demo. These files
' copyright Don Dickinson and are redistributed with permission.
' To compile: FIle ddoc_p32.INC must be available. This file available only to licensed users
' of ddoc Print and Preview. License Information available at http://dickinson.basicguru.com/



#COMPILE EXE
#DEBUG ERROR ON
#REGISTER NONE
' ==[Resource if needed ] =============
' #RESOURCE "XXXXX.PBR"

' for source code bundler to not forget the required BMP files...and a demo report
' file showing both automatic and forced page advance.
#IF 0
   #INCLUDE "watermark.bmp"
   #INCLUDE "logo.bmp"
   #INCLUDE "ddoc_demo.exe"                ' this executable
   #INCLUDE "C:\Utility\DLLs\ddoc.exe"     ' ddoc runtime files
   #INCLUDE "C:\Utility\Dlls\ddoc32.dll"
#ENDIF
'=====[Windows API Header Files] ============================
'  If you don't need all of the functionality supported by these APIs
'  (and who does?), you can selectively turn off various modules by putting
'  the following constants in your program BEFORE you #include "win32api.inc":
'
'  %NOGDI = 1     ' no GDI (Graphics Device Interface) functions
'  %NOMMIDS = 1   ' no Multimedia ID definitions

%NOMMIDS = 1
#INCLUDE "WIN32API.INC"
#INCLUDE "COMDLG32.INC"   ' OpenFileDialog in here
'==[End Windows API Header Files]============================

'=====[GLOBAL datanames] ====================================

' constants for this program
$AppTitle = "ddoc Print and Preview Watermark Demo"
' control IDs
%ID_FileName           = 101&
%ID_BROWSE             = 102&
%ID_Print              = 103&
%ID_Exit               = 104&

DECLARE CALLBACK FUNCTION DialogProc () AS LONG

CALLBACK FUNCTION DialogProc()
 LOCAL wText AS STRING

    IF CBMSG = %WM_INITDIALOG THEN
        CONTROL SET FOCUS CBHNDL, %ID_Filename
        FUNCTION = 1
    ELSEIF CBMSG = %WM_COMMAND THEN
        SELECT CASE CBCTL
          CASE %ID_Print
              CONTROL GET TEXT CBHNDL, %ID_FILENAME TO wTExt
              IF DIR$(wText) <> "" THEN
                  CALL PrintTextFile (wText)
              END IF
             FUNCTION = 0: EXIT FUNCTION
          CASE %ID_BROWSE
               IF ISTRUE OpenFileDialog (CBHNDL,_              ' parent
                                   "Select File To Print", _    'caption
                                   wText, _                    ' filespec
                                   CURDIR$,_                   ' initial dir
                                   "*.*", _                    ' filter
                                   "", _                       ' def extension
                                   %OFN_FILEMUSTEXIST) THEN    ' flags

                     CONTROL SET TEXT CBHNDL, %ID_FILENAME, wText
              END IF
              EXIT FUNCTION

          CASE %ID_EXIT
            DIALOG END CBHNDL, 0
            FUNCTION = 0: EXIT FUNCTION
         END SELECT
    END IF
END FUNCTION

'------------------------------------------------------------------------------
'  PRINTING SUPPORT
' %DDOC_NOHANDLES is the first equate defined in DDOC_P32.INC

#IF NOT %DEF (%DDOC_NOHANDLES)
  #INCLUDE "DDOC_P32.INC"
#ENDIF

$DDOC_NEWPAGE = ".PAGE."   ' << will be added to force a page break
$LOGO_FILE_NAME       =  "Logo.bmp"
$WATERMARK_FILE_NAME  =  "watermark.bmp"
' new file from Roy Cline 3/4/03
'$WATERMARK_FILE_NAME  =  "roycline#1.bmp"
'$WATERMARK_FILE_NAME  =  "ppps_white_watermark_180x90.bmp"
'$WATERMARK_FILE_NAME  =  "pppsabr3_wm.bmp"

FUNCTION PrintTextFile (sFileName AS STRING) AS LONG
' assumes input file is standard text file, CRLF line termination

LOCAL hFile AS LONG, Buffer AS STRING, E AS LONG         ' for file access
LOCAL I AS LONG, FirstPageDone  AS LONG
LOCAL LinesPerPage AS LONG, LinesThisPage AS LONG
LOCAL hDoc AS LONG
LOCAL xPos AS SINGLE, yPos AS SINGLE, LineHeight AS SINGLE
LOCAL szFont AS ASCIIZ * 48, FontSize AS SINGLE
LOCAL PaperSize AS INTEGER, Orientation AS INTEGER, wBin AS INTEGER
LOCAL TextAlign AS INTEGER
LOCAL TopMargin AS SINGLE
LOCAL EndDocOptions AS LONG
' for DDOC graphics/Logo support...
LOCAL hLogo AS LONG, hWM AS LONG
LOCAL szLogoFile AS ASCIIZ * %MAX_PATH, szWatermarkFile AS ASCIIZ * %MAX_PATH
LOCAL wmX AS SINGLE, wmY AS SINGLE
LOCAL ForceNewPage   AS LONG

' set the basic parameters for printing
 szFont         = "Courier New"
 FontSize      = 10!
 TextAlign     = %DDOC_LEFT
 wBin          = %DDOC_BIN_AUTO
 Orientation   = %DDOC_PORTRAIT
 PaperSize     = %DDOC_PAPER_LETTER
 LineHeight    = 1/6         ' 6 lpi
 TopMargin     = .5          ' 1.0 = one inch
 XPos          = .5          ' 1/2" margin on left
 LinesPerPage  = 60
 szLogoFile      =  $LOGO_FILE_NAME
 szWaterMarkFile =  $WATERMARK_FILE_NAME


 LinesThisPage = 999       ' force page advance on first print
 FirstPageDone = %FALSE
 LOCAL dpStartDocOPtions AS LONG
       dpStartDocOptions = %DDOC_FASTTEXT OR %DDOC_ALLOWSAVE

   ' =============================================================
   '  GET A DOCUMENT HANDLE AND SET UP THE GRAPHICS HANDLES
   ' =============================================================

    hDoc = dpStartDoc (0,_                         ' reserved
           "Tal Systems DDOC Demo",_  ' Print job or view title
            "",_                      ' internal file name, null= ddoc creates *.TMP file
            %DDOC_INCH, _             ' UOM
            PaperSize, _              ' PaperSize
            Orientation, _            ' Orientation
            wBin, _                   ' bin
            dpStartDocOptions)        ' options. FastText = vanilla

    IF hDOC <= 0 THEN
         MSGBOX "Error on dpStartDoc=" & STR$(hDoc)
         FUNCTION = -1
         EXIT FUNCTION
    ELSE
     '   CALL  dpSpecifyPrinter(hDoc,szPrinterName)   ' dpspecify printer is SUB
      ' get a ddoc graphics handle for the logo
        hLogo = dpAddGraphic (hDoc, szLogoFile)
        hWm = dpAddGraphic (hDoc, szWaterMarkFile)
        GOSUB ddPrintSetTextFont
        GOSUB PrintPageHeader
        GOSUB PrintPageNos
        GOSUB ddPrintSetTextFont
        LinesThisPage =  8
        ForceNewPage  =  %FALSE
    END IF

    '  =========================================='
    '         OPEN THE FILE WITH THE REPORT TEXT
    '   =========================================='

    hFile = FREEFILE
    OPEN sFileName FOR INPUT AS hFile
    E = ERRCLEAR
    IF E <> 0 THEN
       FUNCTION = E
       MSGBOX "Error on OPEN of file '" & sFileName & "' in PrintReportFile:" & STR$(E), %MB_APPLMODAL OR %MB_ICONSTOP, "Provider Payment Partner Error"
       EXIT FUNCTION
    END IF


                '  =========================================='
                '         MAIN READ TEXT AND PRINT LOOP
                '   =========================================='

   WHILE NOT EOF(hFile)
      LINE INPUT #hFile, Buffer
      IF TRIM$(Buffer) = $DDOC_NEWPAGE THEN
        ForceNewPage = %TRUE
      END IF
      IF (LinesThisPage > LinesPerPage) OR ForceNewPage THEN
          GOSUB NewPageProc
      END IF                           ' if we need a new page
     'Preliminaries out of the way, print this line of text
     ' if this was a forced new page, do not print buffer (contains $DDOC_NEWPAGE)
      IF ForceNewPage THEN
          ForceNewPage = %FALSE     ' reset
      ELSE
         DpText   hDoc, _           ' doc handle
                  xpos, _           ' Xpos
                  yPos, _           ' ypos
                  TextAlign, _          'alignment
                  BYCOPY Buffer                ' Text
         INCR LinesThisPage
         YPos = Ypos + LineHeight
      END IF
   WEND
   CLOSE #hFile  ' close input file now.

   GOSUB DDPrintSetHeaderFont
   Ypos = YPos + LineHeight
   Buffer = "** END OF REPORT **
   dpText   hDoc, 2.0, Ypos, TextAlign, BYCOPY Buffer

  ' =============================================================='
  '          SEND DOCUMENT TO PRINTER OR PREVIEWER
  ' ==============================================================
  'Close out the ddoc document and send to viewer

    EndDocOptions = %DDOC_END_DELETE OR %DDOC_END_VIEW
    DpEndDoc hDoc, EndDocOPtions
   ' %DDOC_END_DELETE = Kill the ddoc workfile.
        EXIT FUNCTION

DDPrintSetTextFont:
   ' Set the font
    dpFont    hdoc,_                    ' document handle
              %DDOC_FONTBOLD, _         ' font characteristics
              FontSize,_                ' font size in points
              0, _                      ' font color (view only? NOPE, Printer, too!) 0 = black
              szFont                    ' Font Name
    RETURN

DDPrintSetHeaderFont:
    ' Set the font
     dpFont  hdoc,_                   ' document handle
             %DDOC_FONTBOLD, _        ' font characteristics
             16!,_                     ' font size in points
              0, _                      ' font color (view only? NOPE, Printer, too!) 0 = black
             szFont                    ' Font Name

    RETURN

   LOCAL hdrBuff AS STRING
PrintPageNos:
  hdrbuff = "Page "
  dpText       hDoc, 6.25, TopMargin, TextAlign, BYCOPY hdrbuff
  dpPageno     hDoc, 7.0!, TopMargin, TExtAlign, BYVAL 0&
  hdrbuff =    "of "
  dpText       hDoc, 7.30!, TopMargin, TextAlign, BYCOPY hdrbuff
  dpPageCount  hDoc, 7.80!, TopMargin, TExtAlign, BYVAL 0&
   RETURN

' Used only for pages 2-n
NewPageProc:
    dpNewPage  hDoc,   _
               PaperSize,   _
               Orientation, _
               wBin
    Ypos = TopMargin
    GOSUB PrintPageHeader
    GOSUB PrintPageNos
    GOSUB ddPrintSetTextFont   ' restore back to normal text font


    RETURN

PrintPageHeader:

     GOSUB ddPrintSetHeaderFont
     GOSUB PrintWatermark
     GOSUB PrintLogo
    ' Print Page Header...provider's name across the top with dpText
     yPos = TopMargin
     FOR I = 1 TO 3   ' for each line of text header..
         SELECT CASE I
            CASE 1
               hdrBuff = "First Line of Page Header"
            CASE 2
               hdrbuff = "Second Line of Page Header"
            CASE 3
              hdrbuff = "Run Date and Time or Whatever
         END SELECT
         DpText   hDoc, _           ' doc handle
                   2.5, _                 ' Xpos  TO RIGHT OF LOGO
                   yPos, _                ' ypos
                   TextAlign, _           ' alignment
                   BYCOPY hdrbuff         ' Text
         INCR LinesThisPage
         YPos = Ypos + LineHeight + .2    ' add .2 to allow for larger font being used
     NEXT
     LinesThisPage = 10
     Ypos = Ypos + 2 * (LineHeight + .2)  ' clear the LOGO area
  RETURN

' dpDrawGraphic  hDoc, hBmp, upperleftX, upperleftY, lowerright x, lowerrightY
PrintLogo:
' called after font is set on each page
  dpDrawGraphic  hDoc, hLogo, .5, .5, 2, 2
  RETURN
PrintWatermark:
   FOR  wmX = .5 TO 6.5 STEP 3.0
      FOR wmY = .5 TO  9.5 STEP 3.0
         dpDrawGraphic hDoc, hWm, wmx, wmy, wmx+ .50!, wmy+0.50!
       NEXT
    NEXT
    RETURN

END FUNCTION


FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

    LOCAL  hDlg AS LONG, Result AS LONG
    ' Create our primary GUI
    DIALOG NEW 0, $AppTitle, 77, 22, 210, 110, _
        %DS_CENTER OR %WS_CAPTION OR %WS_SYSMENU, 0 TO hDlg
    ' Abort if the dialog could not be created
    IF hDlg = 0 THEN EXIT FUNCTION  ' Error occurred

    CONTROL ADD LABEL,    hDlg, -1,            "File to Print",  10, 10 ,110, 12,  %SS_LEFT OR %WS_VISIBLE OR %WS_CHILD
    CONTROL ADD TEXTBOX,  hDlg, %ID_FILENAME,   "",              10, 26, 160, 14
    ' Add the command buttons
    CONTROL ADD BUTTON,    hDlg, %ID_Browse,  "&Browse",    120,  10, 40, 14
    CONTROL ADD BUTTON,   hDlg, %ID_EXIT ,    "&Quit",      130,  75, 40, 14
    CONTROL ADD BUTTON,   hDlg, %ID_PRINT,    "&Print",      70,  46, 40, 14
    ' at least give myself nominal credit:
    CONTROL ADD LABEL,    hDLG, -1,           "Demo Program Courtesy Michael Mattias", 5, 94, 180, 12, %SS_CENTER OR %WS_VISIBLE OR %WS_CHILD

    DIALOG SHOW MODAL hDlg, CALL DialogProc TO Result

    FUNCTION = 0

END FUNCTION


'/// END OF DEMO PROGRAM ///
